package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.SceneLoader;
import de.hdm_stuttgart.battlearena.Controller.Utilities.ScreenDimensionCalculator;
import de.hdm_stuttgart.battlearena.Controller.Enum.GameMode;
import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Controller.Enum.PlayerMode;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityClass;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityFactory;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityType;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectStatus;
import de.hdm_stuttgart.battlearena.Model.Entity.IEntity;
import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;
import de.hdm_stuttgart.battlearena.Model.Map.Biom;
import de.hdm_stuttgart.battlearena.Model.Map.TileManager;

import javafx.animation.AnimationTimer;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Rectangle2D;
import javafx.scene.Node;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;

import javafx.scene.control.Label;
import javafx.scene.effect.GaussianBlur;
import javafx.scene.layout.*;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.stage.Screen;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.net.URL;
import java.util.*;
import java.util.stream.Collectors;

public class GameSceneController implements Initializable {

    private static final Logger log = LogManager.getLogger(GameSceneController.class);
    @FXML
    private BorderPane gameScene;
    @FXML
    private Canvas canvas2D;
    @FXML
    private Rectangle playerHealth, enemyHealth;
    @FXML
    private Pane playerPane, enemyPane;
    @FXML
    private Label playerHp, enemyHp, time, round;
    @FXML
    private StackPane stackPane;
    private final Pane pause = new SceneLoader().loadScene("Pause");

    private boolean wasPaused;
    Screen screen = Screen.getPrimary();
    Rectangle2D visualBounds = screen.getVisualBounds();
    final private double healthBarWidth = screen.getBounds().getWidth() / 5;
    ScreenDimensionCalculator screenCalculator = new ScreenDimensionCalculator();

    private GraphicsContext graphicsContext2D;

    InputHandler inputHandler = InputHandler.getInstance();

    RuntimeInfo runtimeInfo = RuntimeInfo.getInstance();



    GameMode gameMode = runtimeInfo.getGameMode();

    IEntity player;
    IEntity enemy;

    EntityClass playerOneClass = runtimeInfo.getPlayerOneClass();
    EntityClass playerTwoClass = runtimeInfo.getPlayerTwoClass();

    TileManager tileManager;

    //map data


    int horizontalTileCount = 18;
    int verticalTileCount = 18;
    int diffTileCount = 30;
    int scaledTileSize = 48;
    private final Biom biom = runtimeInfo.getMapBiom();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        double diagonalInches = screenCalculator.calculateDiagonalInches(visualBounds.getWidth(),
                visualBounds.getHeight(), screen.getDpi());
//        for some reason scene builder overwrites css fonts bruh
        playerHp.setFont(Font.loadFont(getClass().getResourceAsStream("/fonts/StarshipShadow.ttf"), 55));
        enemyHp.setFont(Font.loadFont(getClass().getResourceAsStream("/fonts/StarshipShadow.ttf"), 55));
        playerHp.setText("100%");
        playerHp.setMinWidth(120);
        playerHp.setMaxWidth(120);
        enemyHp.setText("100%");
        enemyHp.setMinWidth(120);

//        stopping the VBox from shrinking when health bar gets smaller
        playerPane.setMinWidth(healthBarWidth);
        playerPane.setMaxWidth(healthBarWidth);
        enemyPane.setMinWidth(healthBarWidth);
        enemyPane.setMaxWidth(healthBarWidth);

        playerHealth.setWidth(healthBarWidth);
        enemyHealth.setWidth(healthBarWidth);

//        make it responsive
        stackPane.heightProperty().addListener((observableValue, oldValue, newValue) -> {
            canvas2D.setScaleY(newValue.doubleValue() / 870);
            canvas2D.setScaleX(newValue.doubleValue() / 870);
        });

        System.out.println(stackPane.heightProperty());
        System.out.println("dpi: " + screen.getDpi());

        System.out.println("inches: " + diagonalInches);

        graphicsContext2D = canvas2D.getGraphicsContext2D();
        graphicsContext2D.setImageSmoothing(false);

        player = EntityFactory.createEntity(EntityType.PLAYER, graphicsContext2D, inputHandler,
                playerOneClass, this, PlayerMode.PLAYER_ONE);
        if (gameMode == GameMode.NETWORK) {
            enemy = EntityFactory.createEntity(EntityType.NETWORK_PLAYER_TWO, graphicsContext2D, inputHandler,
                    playerTwoClass, this, PlayerMode.PLAYER_TWO);
        } else {
            enemy = EntityFactory.createEntity(EntityType.PLAYER, graphicsContext2D, inputHandler,
                    playerTwoClass, this, PlayerMode.PLAYER_TWO);
        }

        tileManager = new TileManager(graphicsContext2D, diffTileCount, horizontalTileCount,
                verticalTileCount, runtimeInfo.getMapDataGame(), biom);




        runtimeInfo.setGameState(GameState.PLAYING);

        AnimationTimer gameLoop = new AnimationTimer() {
            @Override
            public void handle(long l) {
                if (!inputHandler.isPause()) {
                    graphicsContext2D.clearRect(0, 0, canvas2D.getWidth(), canvas2D.getHeight());
                    renderContent(graphicsContext2D);
                    updateContent();
                    if (wasPaused) {
                        resume();
                    }
                } else {
                    if (!wasPaused) {
                        pauseMenu();
                    }
                }
            }
        };
        gameLoop.start();
        log.debug("Game loop started");
    }

    private void pauseMenu() {
//        setting blur in for loop otherwise all child node will inherit blur from root including pause menu
        for (Node child : gameScene.getChildren()) {
            if (child != stackPane) {
                child.setEffect(new GaussianBlur(20));
            }
        }
        for (Node child : stackPane.getChildren()) {
            if (child != pause) {
                child.setEffect(new GaussianBlur(20));
            }
        }
        stackPane.getChildren().add(pause);
        wasPaused = true;
    }

    private void resume() {
        for (Node child : gameScene.getChildren()) {
            child.setEffect(null);
        }
        for (Node child : stackPane.getChildren()) {
            child.setEffect(null);
        }
        stackPane.getChildren().remove(pause);
        wasPaused = false;
    }


    private void updateContent() {
        player.updateEntity(this);
        enemy.updateEntity(this);
        player.attack(enemy, graphicsContext2D);
        enemy.attack(player, graphicsContext2D);
        updateGameplayObjects();
        player.checkGameplayObjectInteraction(player, graphicsContext2D);
        enemy.checkGameplayObjectInteraction(enemy, graphicsContext2D);
        player.placeBomb(player, graphicsContext2D);
        enemy.placeBomb(enemy, graphicsContext2D);
        double playerHP = (double) getPlayer().getHealth() / getPlayer().getMaxEntityHealth();
        double enemyHP = (double) getEnemy().getHealth() / getEnemy().getMaxEntityHealth();
        if (playerHP >= 0) {
            playerHealth.setWidth(healthBarWidth * (playerHP));
            double playerPercentage = playerHP * 100;
            playerHp.setText((int) playerPercentage + "%");
        } else {
            playerHp.setText(0 + "%");
        }
        if (enemyHP >= 0) {
            enemyHealth.setWidth(healthBarWidth * (enemyHP));
            double enemyPercentage = enemyHP * 100;
            enemyHp.setText((int) enemyPercentage + "%");
        } else {
            enemyHp.setText(0 + "%");
        }
    }

    private void renderContent(GraphicsContext graphicsContext) {
        tileManager.renderMap();
        player.renderEntity(graphicsContext);
        enemy.renderEntity(graphicsContext);
        renderGameplayObjects(graphicsContext);
    }
    
    private void renderGameplayObjects(GraphicsContext graphicsContext) {
        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        for (IEntity gameplayObject : gameplayObjects) {
            gameplayObject.renderEntity(graphicsContext);
        }
    }

    private void updateGameplayObjects() {
        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        List<IEntity> updateList = gameplayObjects.stream().filter(obj -> obj.getOBJECT_STATUS() != ObjectStatus.USED)
                .collect(Collectors.toList());
        runtimeInfo.setGameplayObjects(updateList);
        log.debug("Gameplay objects updated");
    }

    public IEntity getEnemy() {
        return enemy;
    }

    public IEntity getPlayer() {
        return player;
    }

    public TileManager getTileManager() {
        return tileManager;
    }

    public int getScaledTileSize() {
        return scaledTileSize;
    }

    public InputHandler getInputHandler(){
        return inputHandler;
    }
}