package de.hdm_stuttgart.battlearena.Model.Entity;

import de.hdm_stuttgart.battlearena.Controller.GameSceneController;
import de.hdm_stuttgart.battlearena.Model.Map.ITile;

import javafx.geometry.BoundingBox;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

public class CollisionHandler {

    private static final Logger log = LogManager.getLogger(CollisionHandler.class);

    public CollisionHandler() {
        log.debug("CollisionHandler initialized");
    }

    public boolean handleBoxCollision(BoundingBox colliderA, BoundingBox colliderB) {
        return colliderA.getMaxX() > colliderB.getMinX() &&
                colliderA.getMinX() < colliderB.getMaxX() &&
                colliderA.getMaxY() > colliderB.getMinY() &&
                colliderA.getMinY() < colliderB.getMaxY();
    }

    public boolean handleMapCollision(IEntity entity, int scaledTileSize, GameSceneController gameSceneController) {
        int playerLeftX = (int) (entity.getBoxCollider().getMinX());
        int playerRightX = (int) (entity.getBoxCollider().getMinX() + entity.getBoxCollider().getWidth());
        int playerTopY = (int) (entity.getBoxCollider().getMinY());
        int playerBottomY = (int) (entity.getBoxCollider().getMinY() + entity.getBoxCollider().getHeight());

        int leftColumn = playerLeftX / scaledTileSize;
        int rightColumn = playerRightX / scaledTileSize;
        int topRow = playerTopY / scaledTileSize;
        int bottomRow = playerBottomY / scaledTileSize;

        int nextPlayerTop = (playerTopY - entity.getEntitySpeed()) / scaledTileSize;
        int nextPlayerBottom = (playerBottomY + entity.getEntitySpeed()) / scaledTileSize;
        int nextPlayerLeft = (playerLeftX - entity.getEntitySpeed()) / scaledTileSize;
        int nextPlayerRight = (playerRightX + entity.getEntitySpeed()) / scaledTileSize;

        int[][] tileMap = gameSceneController.getTileManager().getTileMap();
        ITile[] tileSet = gameSceneController.getTileManager().getTileSet();
        int tileOne;
        int tileTwo;

        switch (entity.getEntityDirection()) {
            case UP:
                tileOne = tileMap[nextPlayerTop][leftColumn];
                tileTwo = tileMap[nextPlayerTop][rightColumn];
                return tileSet[tileOne].getCollision() && tileSet[tileTwo].getCollision();
            case DOWN:
                tileOne = tileMap[nextPlayerBottom][leftColumn];
                tileTwo = tileMap[nextPlayerBottom][rightColumn];
                return tileSet[tileOne].getCollision() && tileSet[tileTwo].getCollision();
            case LEFT:
                tileOne = tileMap[topRow][nextPlayerLeft];
                tileTwo = tileMap[bottomRow][nextPlayerLeft];
                return tileSet[tileOne].getCollision() && tileSet[tileTwo].getCollision();
            case RIGHT:
                tileOne = tileMap[topRow][nextPlayerRight];
                tileTwo = tileMap[bottomRow][nextPlayerRight];
                return tileSet[tileOne].getCollision() && tileSet[tileTwo].getCollision();
            default:
                return true;
        }
    }

}